/*
    Kommando Menu Routinen

    Dieses Modul enthält Routinen, welche aus dem Auswahlmenü
    aufgerufen werden, nachdem der Kommandostring ausgewertet
    wurde.

    (C) Christian Julius 11.2014
    christian.julius@gmx.de

*/
#include <asm/z80/features.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <stdint.h>
#include <ctype.h>
#include <string.h>
#include "main.h"
#include "mcurses.h"
#include "hexedit.h"
#include "driver.h"
#include "command.h"
#include "sd.h"

#pragma disable_warning 196         // Const Zeiger verloren ????
#pragma disable_warning 85          // Variablen nicht benutzt

extern __at (RAM_START) unsigned char ram[RAM_SIZE];          // Bereich des RAM
extern __at (ROM_START) unsigned char rom[ROM_SIZE];          // ROM
extern __at (ROM_START) unsigned char memory[0xffff];         // Kompletter Speicher

extern struct system_type sys;             // System Infos
extern struct cl_type cl;                   // Eingabepuffer mit allen Parametern

///////////////////////////////////////////////////////////
// Zerlegt den Kommandostring in Befehl und Parameter
// Befehl in cmd[]
// Parameter in param[]
// Eingabe: Call by reference struct auf cl_type
// Rückgabe: Anzahl Parameter


/*
struct cl_type {
       char inbuf[CMDBUFSIZE];              // Eingabepuffer
       char cmd[CMD_LEN];                   // Kommandostring
       char param[MAX_PARAMS][PARAM_LEN];   // Parameterliste (Strings)
       char no_params;                      // Anzahl übergebener Parameter
};
*/

// Erzeugt ein CR/LF
void nextline()
{
    uint8_t ypos,xpos;

    getyx(ypos,xpos);
    move(ypos+1,0);
}

// Koordinaten x,y unten links
void gotoxy(uint8_t x, uint8_t y)
{
    if ((x<=LINES) & (y<=COLS))
        move(y,LINES-x);
}


// Statuszeile unten ausgeben
/*
void statusline(char* zeile)
{
    #define STATYPOS    23
    char xpos,ypos;

    getyx(ypos,xpos);           // Alte Pos speichern
    move(STATYPOS,0);          // Statuszeile
    deleteln();
    addstr("Z80 > ");
    addstr(zeile);
    move(ypos,xpos);
}
*/

// Gibt Text aus und wartet auf <Return>
void press_return()
{
    addstr(" Press <RETURN>");
    while(getchar()!=0x0d);
}

// FIXXXXME!
void menu_getad()
{
    uint8_t foo = 0;
    uint8_t old = 0;
    
     addstr("\r\nZ80 > Reading ADC... (CR to stop)");
     curs_set(0);
     while (keypress()!=0x0d)
     {
         foo = adc_read();
         if (foo != old) {
            old = foo;
            move(menu_entries+4,6);
            addstr("ADC:      ");
            move(menu_entries+4,12);
            addint((uint8_t)adc_read());
            delay(3000);
            }
      }
      curs_set(2);
}

// Lädt ein ausführbares Programm von der Uart
// oder SD Karte herunter

char menu_load_file()
{
    uint8_t xpos,ypos;

    addstr("\r\nZ80 > Waiting for PC Data!");
    getyx(ypos,xpos);
    move(ypos+2,0);
    if (ihx_decode()) {
        sys.f_prog_loaded = 1;
        addstr("\r\nIHX8 data loaded.");
        return(1);
    }
    else {
        sys.f_prog_loaded = 0;
        addstr("\r\nIHX8 Checksum Error!");
        return (0);
    }

    return (1);
}

// Speichert ein Programm oder Daten auf SD Karte

char menu_save_file()
{
    uint8_t descr[40];
    uint8_t i;

    // Eingabeprüfung
    if (cl.no_params<1) {
        addstr("\r\nZ80 > Filename missing!");
        press_return();
        return (0);
    }

    if (strlen(cl.param[0])>11) {
        addstr("\r\nZ80 > Filename too long (11)!");
        press_return();
        return (0);
    }

    // Aus Parameter 1-n evtl.Beschreibung zusammen bauen
    // in descr[40]
    clear_buffer(descr,sizeof(descr));
    if (cl.no_params > 1)
    {
        for (i=1;i<cl.no_params;i++)
        {
            if (strlen(descr)<MAX_DESCR_SIZE) {
                strcat(descr,cl.param[i]);
                strcat(descr," ");
            }
        }
    }
    else
        strcpy(descr,"<none>");

    addstr("\r\nZ80 > Saving File to SD...");
    sd_save_prog(cl.param[0],descr);

    //press_return();
    return (1);


}

// Startet ein Programm im RAM
void menu_run_file()
{
    if (sys.f_prog_loaded) {
        __asm
              jp RAM_START
        __endasm;
     }
     else  {
         addstr("\r\nZ80 > No userprogram loaded!");
         press_return();
     }
}

void menu_del_file()
{

    press_return();


}

// Listet die SD karte auf
void menu_cat_sd()
{


    press_return();
}

// Loescht das RAM
void menu_clear_ram()
{
    uint16_t count;

    addstr("\r\nZ80 > Erasing RAM...");
    for (count=RAM_START;count<DATA_START;count++)
            memory[count]=0;

    addstr("\r\nZ80 > RAM erased!");

    press_return();
}


// Setzt einen Port auf einen Wert
void menu_setport()
{

    press_return();



}

// Prüft das RAM auf Fehler
char menu_test_ram()
{

    uint8_t linecount, pattern,schalt;
    uint16_t round,count;

    round = 0;
    linecount = 0;
    schalt = 0;
    SEG7 = 0xff;
    nextline();
    addstr("Z80 > Testing RAM... <Reset to stop>");
    while (1) {
        nextline();
        addstr("Z80 > Testrun: "); addint(round);
        nextline();
        for (pattern = 0x55;pattern<=0xAA;pattern++)
        {
            // Einschreiben
            LedOut(pattern);
            for (count=RAM_START;count<DATA_START;count++)
                memory[count]=pattern;
            // Vergleiche...
            for (count=RAM_START;count < DATA_START;count++)
            {
                if (memory[count]!=pattern) {
                    addstr("ERROR! : "); addint(pattern);
                    SetDigit(0x0e);
                    while (1)
                    {
                       LedOut(0x00);
                       delay(2000);
                       LedOut(0xff);
                       delay(2000);
                    }
                 }
            }
            if (schalt) {
                addstr("*");
            }
            else {
                addstr("-");
            }
            if (linecount++ >= 53)
            {
                linecount = 0;
                nextline();
            }
        }

        addstr(" RAM OK!");
        schalt = ~schalt;
        linecount = 0;
    }

    return (0);

}

