;--------------------------------------------------------------------------
; crt0.s - Generic crt0.s for a Z80
;
; Minimum Z80 System mit Mostek 3801 Multi I/O Chip
;
;  (C)Christian Julius 10.2014
;  christian.julius@gmx.de
;--------------------------------------------------------------------------

    .module crt0            ; Modul Name fr den Linker

    ; Memory Map Adressen der Zeropage (vom Benutzer definierbar)
    ; Definitions

    offset           .equ   0x0000   ; Basisadresse des Codes
    reset_vector     .equ   offset
    stack            .equ   0x0000   ; Stack
    adr_vec_table    .equ   0x40   ; Mode 2 Vektor Tabelle fr STI Mostek Multi I/O
    adr_nmi          .equ   0x66   ;  NMI Reset Vektor
    adr_copyright    .equ   0x80
    ram_prog         .equ   0x4000

    ; Globale Funktionen und Variablen fr das C-Programm

    .globl  _main
    .globl  _nmi_vint       ; Funktion des NMI Handlers
    
    ; Interruptroutinen fr den Mostek MK3801 Multi I/O Baustein
    ; Manual STI Baustein MK3801 Figure 7
    .globl  _int_sti_gpi_0  ; General Purpose Interrupt 0
    .globl  _int_sti_gpi_1  ; General Purpose Interrupt 1
    .globl  _int_sti_gpi_2  ; General Purpose Interrupt 2
    .globl  _int_sti_gpi_3  ; General Purpose Interrupt 3
    .globl  _int_sti_timer_d
    .globl  _int_sti_timer_c
    .globl  _int_sti_gpi_4  ; General Purpose Interrupt 4
    .globl  _int_sti_gpi_5  ; General Purpose Interrupt 5
    .globl  _int_sti_timer_b
    .globl  _int_sti_transmit_error
    .globl  _int_sti_transmit_buffer_empty
    .globl  _int_sti_receive_error
    .globl  _int_sti_receive_buffer_full
    .globl  _int_sti_timer_a
    .globl  _int_sti_gpi_6  ; General Purpose Interrupt 6
    .globl  _int_sti_gpi_7  ; General Purpose Interrupt 7

    .globl l__INITIALIZER
    .globl s__INITIALIZER
    .globl s__INITIALIZED

    .globl _letzte_code_adresse
    .globl _erste_code_adresse

    ; -------------------------------------------------------------

    .area   _HEADER (ABS)
    ; Reset vector bei Kaltstart und Sprung ins User Programm
    .org    reset_vector
    _erste_code_adresse:
     jp init

    ; Tabelle der RST Vektoren fr Software Interrupts
     .org 0x08
     jp ram_prog+0x0008
     .org 0x10
     jp ram_prog+0x0010
     .org 0x18
     jp ram_prog+0x0018
     .org 0x20
     jp ram_prog+0x0020
     .org 0x28
     jp ram_prog+0x0028
     .org 0x30
     jp ram_prog+0x0030
     .org 0x38
     jp ram_prog+0x0038


    ;///////////////////////////////////////////////////////////
    ; Tabelle der Mode 2 Int Vektoren auf die Handler Funktionen im C Modul
    ; Reihenfolge beachten !!! Siehe Manual Mostek 3801, Interrupt Tabelle
    .org adr_vec_table
    .dw (_int_sti_gpi_0)
    .dw (_int_sti_gpi_1)
    .dw (_int_sti_gpi_2)
    .dw (_int_sti_gpi_3)
    .dw (_int_sti_timer_d)
    .dw (_int_sti_timer_c)
    .dw (_int_sti_gpi_4)
    .dw (_int_sti_gpi_5)
    .dw (_int_sti_timer_b)
    .dw (_int_sti_transmit_error)
    .dw (_int_sti_transmit_buffer_empty)
    .dw (_int_sti_receive_error)
    .dw (_int_sti_receive_buffer_full)
    .dw (_int_sti_timer_a)
    .dw (_int_sti_gpi_6)
    .dw (_int_sti_gpi_7)

    ;///////////////////////////////////////////////////////////
    ; NMI Interrupt 0x66, muss mit retn abgeschlossen werden
    .org  adr_nmi
    jp _nmi_vint        ; Aufruf Handler im C Modul main.c
    
    .org adr_copyright
    .asciz "Z80/MK3801 System (C) 12.2014 by C.Julius"

    ;///////////////////////////////////////////////////////////
    ; Anordung der Segmente fuer den Linker, ab hier nur relative
    ; Adressen

      .area  _CODE
      .area  _INITIALIZER
      .area  _HOME
      .area  _GSINIT
      .area  _GSFINAL
      .area  _DATA
      .area  _INITIALIZED
      .area  _BSEG
      .area  _BSS
      .area  _HEAP

    ;///////////////////////////////////////////////////////////
    ;; ------- Start des Hauptprogramms nach der Zeropage  -----------

    .area _CODE
    init:

    ld  sp,#stack   ;; Stack an Spitze des RAM legen

    ; Teste ob Userprogramm im RAM vorhanden (nur bei EPROM Version)
    ld a, (ram_prog)   ; Lade Start des RAM 0x2000
    cp #0xC3           ; vergleiche mit "C3" = JP Befehl
    jp Z, ram_prog     ; Ja, dann springe direkt ins Ram hinein

    ; Ram ausnullen (auskommentieren, wenn von PC Upload erfolgt)
    xor     a                   ; clear a and carry
    ld      bc,#(0xffff-ram_prog)  ; ram size left
    ld      hl,#ram_prog
    ld      de,#ram_prog+1
    ld      (hl),a              ; 0 -> (4000)
    ldir                        ; (HL) -> (DE)

    call gsinit     ;; Static Variablen init
    call  _main     ;; Hauptprogramm aufrufen
endlos:
    halt
    jr endlos

   .area  _GSINIT

 gsinit:

    ; Initialisiere globale Variablen im RAM
    ld  bc, #l__INITIALIZER
    ld  a, b
    or  a, c
    jr  Z, weiter
    ld  de, #s__INITIALIZED
    ld  hl, #s__INITIALIZER
    ldir

weiter:

    ; // Hier muss ein ret rein
    .area _GSFINAL
      ret
     ;Markiert das Ende des Programmes
     _letzte_code_adresse:





