/////////////////////////////////////////////////////////
/*
   Minimum Z80 System mit Mostek 3801 Multi I/O Chip
   (C)Christian Julius 10.2014
   christian.julius@gmx.de

   Hauptprogramm

*/
/////////////////////////////////////////////////////////


// --- Standard Header ---
#include <asm/z80/features.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include <ctype.h>
// --- Projekt Header ---
#include "command.h"
#include "mcurses.h"
#include "hexedit.h"
#include "driver.h"
#include "sd.h"
#include "tools.h"
#include "main.h"

//#pragma disable_warning 154
#pragma disable_warning 196         // Const Zeiger verloren ????
#pragma disable_warning 85          // Variablen nicht benutzt

// --------Interne Variablen -------
// Definiert in crt0.s Nicht ndern!
extern char erste_code_adresse[];         // Zeiger auf SDCC Globals
extern char letzte_code_adresse[];

struct system_type sys;             // System Infos
struct cl_type cl;                  // Eingabepuffer mit allen Parametern
uint8_t AD_Value;                    // Aktueller AD Wert (von TA ausgelesen

////////////////////////////////////////////
//      Globale Variablen
/////////////////////////////////////////////


// ----- Variablen des Watchdogs -----
extern uint16_t TA_Reset;
extern uint16_t TA_Timeout;
extern char     TA_Enable;


// Absolute Adressen des gesamten RAMs (ohne Stack) und ROMs
// Diese Array sind Overlays und bentigen keinen Speicherplatz
__at (RAM_START) unsigned char ram[RAM_SIZE];          // Bereich des RAM
__at (ROM_START) unsigned char rom[ROM_SIZE];          // ROM
__at (ROM_START) unsigned char memory[0xffff];         // Kompletter Speicher

// Menu Ausgabe
const char* const menu[] = {
  "Z80 Monitor V1.0 (C) C.Julius 12.2014\r\n",
  "\r\n[load] <sd n>.<uart>.........Load File <n> <source>"
  "\r\n[save] <n>......................... Save File to SD",
  "\r\n[del]  <n>................... Delete File n from SD",
  "\r\n[cat]................................... Catalog SD",
  "\r\n[run].................................. Run Program",
  "\r\n[clear].................................. Clear RAM",
  "\r\n[dump]................................. Hex Monitor",
  "\r\n[ramtest]................................. Test RAM",
  "\r\n[adc].............................. Lies AD Wandler",
  "\r\n[set] <port> <$value>....Set Port A/B/C to <$value>",
  "\r\n[reset]............................. Restart System",
  "\r\n",
  "\r\nuser>"
};

void show_menu()
{
    char i;
    clear();
    // Menu anzeigen
    for (i=0;i<menu_entries;i++)
      addstr(menu[i]);

    move(menu_entries+1,6);
}



//#define DEBUG_COMMAND

// Rueckgabe 2 = String zu lang, 1 = ok, 0 = keine Zeichen
char split_command(struct cl_type* ptr)
{
    char cbuf[CMDBUFSIZE];               // Kopie Eingabepuffer
    char* cptr;                          // Zeiger auf Eingabepuffer
    char j,i;

    // Stringzerlegung (ok!)
    strcpy(cbuf,ptr->inbuf);    // String Kopie erzeugen

    j = 0;
    cptr = strtok(cbuf," ");             // Zeichentrenner und erster Abschnitt
    strcpy(ptr->cmd,cptr);
    while(cptr != 0 && (j < MAX_PARAMS))
    {
        cptr = strtok(NULL," ");
        if (cptr != NULL) {
            if (strlen(cptr)<PARAM_LEN)
                strcpy(ptr->param[j++],cptr);
            else
                return (2);
        }
    }

    ptr->no_params = j;

    #ifdef DEBUG_COMMAND
    // Testweise Ausgabe der Zerlegung
    xprintf("\r\ncmd: %s",ptr->cmd);
    xprintf("\r\nParams: %d",j);
    for (i=0;i < ptr->no_params;i++)
        xprintf("\r\nParam %d : %s",i,ptr->param[i]);
    #endif

    // 1 = ok
    return (1);
}

////////////////////////////////////////////////////////////////////////
//                             M A I N
////////////////////////////////////////////////////////////////////////

void main()
{
    char i,res,foo,old = 0;

   // Sichere die Start- und Endadresse des Codes
    sys.code_start = (uint16_t)erste_code_adresse;
    sys.code_end   = (uint16_t)letzte_code_adresse;
    sys.code_length = letzte_code_adresse-erste_code_adresse;

    // Initialisierung

    sys.f_TA_Enable = false;
    InitHardware();                 // Hardware Init
    interrupts(true);               // Globale Ints ein
    init_buffer();                  // Buffer Init
    rx_full_enable(true);           // Uart RX Ints einschalten
    tx_full_enable(true);           // Uart RX Ints einschalten
    ta_full_enable(true);
    init_spi();                     // SPI vorbereiten

    // ---
    SEG7 = 0xff;
    initscr();                    // Bildschirm vorbereiten
    curs_set(2);
    attrset(A_NORMAL);

    while (1) {
        do {
            show_menu();
            // Leerstrings ignorieren
            do {
                getnstr(cl.inbuf,CMDBUFSIZE-1);
            } while (strlen(cl.inbuf)==0);

            // String splitten
            res = split_command(&cl);
            // und Test auf Maximallaenge der Wrter
            if (res == 2) {
                addstr("\r\nZ80 > Error!: Only 12 chars per word!");
                press_return();
            }
        } while (res != 1);

        ////////////////////////////////////////////////////////
        // Hex Editor
        ////////////////////////////////////////////////////////
        if (strcmp(cl.cmd,"dump")==0)
        {
                hexedit (0);
        }
        ////////////////////////////////////////////////////////
        // Lade Userprogramm
        ////////////////////////////////////////////////////////
        else if (strcmp(cl.cmd,"load")==0)
        {
            if (menu_load_file())
               sys.f_prog_loaded = true;
            else 
               sys.f_prog_loaded = false;
            
            press_return();
        }
        ////////////////////////////////////////////////////////
        // Speichere Userprogramm
        ////////////////////////////////////////////////////////
        else if (strcmp(cl.cmd,"save")==0)
        {
            menu_save_file();
        }
        ////////////////////////////////////////////////////////
        // Loesche File von SD Karte
        ////////////////////////////////////////////////////////
        else if (strcmp(cl.cmd,"del")==0)
        {
            menu_del_file();
        }
        ////////////////////////////////////////////////////////
        // Katalog SD Karte
        ////////////////////////////////////////////////////////
        else if (strcmp(cl.cmd,"cat")==0)
        {
            menu_cat_sd();
        }
        ////////////////////////////////////////////////////////
        // Starte Userprogramm
        ////////////////////////////////////////////////////////
        else if (strcmp(cl.cmd,"run")==0)
        {
            menu_run_file();
         }
        ////////////////////////////////////////////////////////
        // Clear memory
        ////////////////////////////////////////////////////////
        else if (strcmp(cl.cmd,"clear")==0)
        {
           menu_clear_ram();
           sys.f_prog_loaded = 0;
        }
        ////////////////////////////////////////////////////////
        // Ad Wandler auslesen
        ////////////////////////////////////////////////////////
        else if (strcmp(cl.cmd,"adc")==0)
        {
            menu_getad();
        }
        ////////////////////////////////////////////////////////
        // System Reset
        ////////////////////////////////////////////////////////
        else  if (strcmp(cl.cmd,"reset")==0)
        {
            addstr("\r\nZ80 > Reset System...");
            delay(25000);
            __asm
                di
                jp 0x0000
            __endasm;
        }
        ////////////////////////////////////////////////////////
        // RAM Dauertest
        ////////////////////////////////////////////////////////
        else  if (strcmp(cl.cmd,"ramtest")==0)
        {
            menu_test_ram();
        }

        else {
            addstr("\r\nZ80 > Unknown Command!");
            press_return();
        }

    }

    while(1);

    //endwin ();

}



