#ifndef main_h
    #define main_h

#define  HIGH     1
#define  LOW      0

// Spezielle Module einbinden

//#define USE_SIMPLE_RANDOM
//#define USE_MERSENNE_TWISTER


// Memory (bei ROM Version hier 0x0000 eintragen)
#define CODE_START  0x0000          // Fr I-Register Interrupt Tabelle
#define STACK_SIZE  0x01ff          // 512 Bytes Stack vorsehen
#define DATA_START  0xF800

#define ROM_START   0x0000          // Startadresse des ROM
#define ROM_END     0x3fff
#define ROM_SIZE    (ROM_END-ROM_START)

#define RAM_START   0x4000
#define RAM_END     0xFFFF
#define RAM_SIZE    (RAM_END-RAM_START)

#define CMDBUFSIZE  80          // Groesse des Komandozeilen Puffers
#define CMD_LEN     16          // Max Laenge des Kommandos
#define MAX_PARAMS  8           // Max Anzahl Parameter
#define PARAM_LEN   16          // Max Lnge eines Parameters

#define menu_entries   13


// Infos ber den Systemzustand
struct system_type {

        uint8_t  adc_val;       // Aktueller AD Wert (von TA ausgelesen
        _Bool    f_adc_ready;   // Flag, 1=ADC ist fertig (gesetzt von ISR)
        _Bool    f_prog_loaded; // Flag: 1=Userprogramm wurde geladen

        uint16_t TA_Timeout;    // Timeout zeit
        _Bool   f_TA_Enable;    // true = Watchdog EIN
        _Bool   f_WDT;          // muss auf false gesetzt werden vom Hauptprogramm, sonst reset

        uint16_t code_start,    // Adressen fr Start und Ende des Programms
                 code_end,
                 code_length;
};


// Kommando Zeilen Auswertung
struct cl_type {
       char inbuf[CMDBUFSIZE];              // Eingabepuffer
       char cmd[CMD_LEN];                   // Kommandostring
       char param[MAX_PARAMS][PARAM_LEN];   // Parameterliste (Strings)
       char no_params;                      // Anzahl bergebener Parameter
};

// Makros
#define LedOut(x)         PIO8255_PORT_A  = ~x

// Bitmanipulation
#define sbi(a, b) (a) |= (1 << (b))
#define cbi(a, b) (a) &= ~(1 << (b))

// Alternativ:
#define BitSet(arg,val) ((arg) |= (val))
#define BitClr(arg,val) ((arg) &= ~(val))
#define BitFlp(arg,val) ((arg) ^= (val))
#define BitChk(arg,val) (((arg) & (val)) == (val))
#define BitVal(arg,bits,val) (!(((arg) & (bits)) ^ (val)))


#endif

